@@ -8,13 +8,12 @@
 #' @param description The text patteren to find a pin.
 #' @param type The type of pin being stored.
 #' @param metadata A list containing additional metadata describing the pin.
+#' @param retrieve Should the pin be retrieved after being created? Defaults to \code{TRUE}.
 #' @param ... Additional parameteres.
 #'
-#' @rdname custom-pins
-#'
 #' @export
 #' @rdname custom-pins
-board_pin_store <- function(board, path, name, description, type, metadata, extract = TRUE, ...) {
+board_pin_store <- function(board, path, name, description, type, metadata, extract = TRUE, retrieve = TRUE, ...) {
   board <- board_get(board)
   if (is.null(name)) name <- gsub("[^a-zA-Z0-9]+", "_", tools::file_path_sans_ext(basename(path)))[[1]]
   pin_log("Storing ", name, " into board ", board$name, " with type ", type)
@@ -41,7 +40,17 @@ board_pin_store <- function(board, path, name, description, type, metadata, extr
   }
 
   if (identical(zip, TRUE)) {
-    zip(file.path(store_path, "data.zip"), path)
+    find_common_path <- function(path) {
+      common <- path[1]
+      if (all(startsWith(path, common)) || identical(common, dirname(common))) return(common)
+      return(force(find_common_path(dirname(common[1]))))
+    }
+
+    common_path <- find_common_path(path)
+    withr::with_dir(common_path, {
+      zip(file.path(store_path, "data.zip"), gsub(paste0(common_path, "/"), "", path), flags="-q")
+    })
+
     something_changed <- TRUE
   }
   else {
@@ -64,11 +73,20 @@ board_pin_store <- function(board, path, name, description, type, metadata, extr
       }
 
       if (details$something_changed) {
+        copy_or_link <- function(from, to) {
+          if (file.exists(from) && file.info(from)$size >= getOption("pins.link.size", 10^8))
+            fs::link_create(from, file.path(to, basename(from)))
+          else
+            file.copy(from, to, recursive = TRUE)
+        }
+
         if (dir.exists(single_path)) {
-          file.copy(dir(single_path, full.names = TRUE) , store_path, recursive = TRUE)
+          for (entry in dir(single_path, full.names = TRUE)) {
+            copy_or_link(entry, store_path)
+          }
         }
         else {
-          file.copy(single_path, store_path, recursive = TRUE)
+          copy_or_link(single_path, store_path)
         }
 
         something_changed <- TRUE
@@ -91,8 +109,13 @@ board_pin_store <- function(board, path, name, description, type, metadata, extr
     ui_viewer_updated(board)
   }
 
-  pin_get(name, board$name, ...) %>%
-    invisible_maybe()
+  if (retrieve) {
+    pin_get(name, board$name, ...) %>%
+      invisible_maybe()
+  }
+  else {
+    invisible(NULL)
+  }
 }
 
 invisible_maybe <- function(e) {
